package edu.travel.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import edu.travel.adapter.service.commodity.ShopProductAdapter;
import edu.travel.remote.dto.GetProductByTypeDto;
import edu.travel.remote.dto.InsertProductDto;
import edu.travel.remote.dto.SearchProductDto;
import edu.travel.remote.vo.FileWebVo;
import edu.travel.remote.vo.ShopProductVo;
import edu.travel.resp.BaseResponse;
import edu.travel.rpc.RPCBaseResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.List;

/**
 * 商品表(shop_product)表控制层
 *
 * @author xxxxx
 */
@RestController
@RequestMapping("/shopProduct")
public class ShopProductController {
    @Autowired
    private ShopProductAdapter shopProductAdapter;

    /**
     * 首页获取热门类别的热门商品
     *
     * @param param 类型
     * @return {@link BaseResponse }<{@link List }<{@link ShopProductVo }>>
     */
    @GetMapping("/getHotProduct")
    public RPCBaseResponse<Page<ShopProductVo>> getHotProduct(GetProductByTypeDto param) {
        return shopProductAdapter.getHotProduct(param);
    }

    /**
     * 搜索商品
     *
     * @param param 类型
     * @return {@link BaseResponse }<{@link List }<{@link ShopProductVo }>>
     */
    @GetMapping("/searchProduct")
    public RPCBaseResponse<Page<ShopProductVo>> searchProduct(SearchProductDto param) throws IOException {
        return shopProductAdapter.searchProduct(param);
    }


    /**
     * 上传商品图片
     *
     * @param file
     * @return {@link RPCBaseResponse }<{@link String }>
     */

    @PostMapping("/uploadProductImage")
    public RPCBaseResponse<FileWebVo> uploadProductImage(MultipartFile file) {
        return shopProductAdapter.uploadProductImage(file);
    }


    /**
     * 新增商品
     *
     * @param params
     * @return {@link RPCBaseResponse }<{@link Void }>
     */

    @PostMapping("/insertProduct")
    public RPCBaseResponse<Void> insertProduct(@RequestBody InsertProductDto params) {
        return shopProductAdapter.insertProduct(params);
    }

    /**
     * 获取SkuId
     *
     * @return {@link RPCBaseResponse }<{@link String }>
     */

    @GetMapping("/getSkuId")
    public RPCBaseResponse<String> getSkuId() {
        return shopProductAdapter.getSkuId();
    }
}
