package edu.travel.cache.aspects;

import edu.travel.cache.annotation.ClearAndReloadCache;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.Signature;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import java.lang.reflect.Method;
import java.util.Set;

@Aspect
@Component
public class CacheAspects {
    @Autowired
    private RedisTemplate redisTemplate;

    // 定义切点，例如拦截所有service层的方法
    @Pointcut("@annotation(edu.travel.cache.annotation.ClearAndReloadCache)")
    public void cachePointcut() {
    }

    @Around("cachePointcut()")
    public Object cacheAround(ProceedingJoinPoint joinPoint) throws Throwable {
        Signature signature = joinPoint.getSignature();
        Object result = null;
        if (signature instanceof MethodSignature) {
            MethodSignature methodSignature = (MethodSignature) signature;
            Method targetMethod = methodSignature.getMethod();//方法对象
            ClearAndReloadCache annotation = targetMethod.getAnnotation(ClearAndReloadCache.class);
            boolean value = annotation.value();
            String pattern;
            if (value) {
                Set keys;
                if (annotation.prefixFactory()) {
                    keys = getKeys(annotation.cachePrefix());
                    pattern = annotation.cachePrefix();
                } else {
                    keys = getKeys(annotation.cacheSuffix());
                    pattern = annotation.cacheSuffix();
                }
                redisTemplate.delete(keys);
            } else {
                pattern = null;
            }
            result = joinPoint.proceed(joinPoint.getArgs());
            new Thread(() -> {
                try {
                    Thread.sleep(1000);
                    Set keys = getKeys(pattern);
                    redisTemplate.delete(keys);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }).start();
        }
        return result;
    }

    private Set getKeys(String pattern) {
        return redisTemplate.keys("*" + pattern + "*");
    }
}
