package edu.travel.service;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import edu.travel.annotation.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.jdbc.core.JdbcTemplate;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 扩展mbp ServiceImpl
 *
 * @param <M>
 * @param <T>
 * @author 忠畅
 */
public class SysServiceImpl<M extends BaseMapper<T>, T> extends ServiceImpl<M, T> implements IService<T> {
    private ApplicationContext applicationContext;
    private JdbcTemplate jdbcTemplate;

    @Autowired
    public void setApplicationContext(ApplicationContext applicationContext) {
        this.applicationContext = applicationContext;
    }

    @Autowired
    public void setJdbcTemplate(JdbcTemplate jdbcTemplate) {
        this.jdbcTemplate = jdbcTemplate;
    }

    /**
     * 一对一字典查询处理
     *
     * @param queryWrapper
     * @return
     */
    public T getOneLink(Wrapper<T> queryWrapper) {
        T one = super.getOne(queryWrapper);
        if (one == null) {
            return null;
        }
        doOneLink(one);
        return one;
    }

    /**
     * 一对多字典查询处理
     *
     * @param queryWrapper
     * @return
     */
    public List<T> getListLink(Wrapper<T> queryWrapper) {
        List<T> list = super.list(queryWrapper);
        if (list == null || list.isEmpty()) {
            return list;
        }
        doManyLink(list);
        return list;
    }

    /**
     * 分页查询处理
     *
     * @param queryWrapper
     * @param page
     * @return
     */
    public IPage<T> getPageLink(Wrapper<T> queryWrapper, IPage<T> page) {
        IPage<T> iPage = super.page(page, queryWrapper);
        if (iPage == null) {
            return new Page();
        }
        List<T> records = iPage.getRecords();
        doManyLink(records);
        iPage.setRecords(records);
        return iPage;
    }

    /**
     * 一对一字典查询处理
     *
     * @param queryWrapper
     * @return
     */
    public T getOneLinkSQL(Wrapper<T> queryWrapper, Object... params) {
        T one = super.getOne(queryWrapper);
        if (one == null) {
            return null;
        }
        doOneLink(one, params);
        return one;
    }

    /**
     * 一对多字典查询处理
     *
     * @param queryWrapper
     * @return
     */
    public List<T> getListLinkSQL(Wrapper<T> queryWrapper, Object... params) {
        List<T> list = super.list(queryWrapper);
        if (list == null || list.isEmpty()) {
            return list;
        }
        doManyLink(list, params);
        return list;
    }

    /**
     * 分页查询处理
     *
     * @param queryWrapper
     * @param page
     * @return
     */
    public IPage<T> getPageLinkSQL(Wrapper<T> queryWrapper, IPage<T> page, Object... params) {
        IPage<T> iPage = super.page(page, queryWrapper);
        if (iPage == null) {
            return new Page();
        }
        List<T> records = iPage.getRecords();
        doManyLink(records, params);
        iPage.setRecords(records);
        return iPage;
    }

    /**
     * 多对多查询字典处理
     *
     * @param list
     */
    private void doManyLink(List<T> list) {
        for (T many : list) {
            doOneLink(many);
        }
    }

    /**
     * 多对多查询字典处理
     *
     * @param list
     */
    private void doManyLink(List<T> list, Object... params) {
        for (T many : list) {
            doOneLink(many, params);
        }
    }

    /**
     * 单个字典处理
     *
     * @param one
     */
    private void doOneLink(T one) {
        Class<?> clazz = one.getClass();
        Field[] declaredFields = clazz.getDeclaredFields();
        for (Field field : declaredFields) {
            field.setAccessible(true);
            doLinkOneToOne(one, field);
            doLinkOneToMany(one, field);
            doConst(one, field);
            doManyToMany(one, field);
            try {
                doLinkSQL(one, null, field);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    private void doOneLink(T one, Object... params) {
        Class<?> clazz = one.getClass();
        Field[] declaredFields = clazz.getDeclaredFields();
        for (Field field : declaredFields) {
            field.setAccessible(true);
            doLinkOneToOne(one, field);
            doLinkOneToMany(one, field);
            doConst(one, field);
            doManyToMany(one, field);
            try {
                doLinkSQL(one, params, field);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * 常量字典处理
     *
     * @param one
     * @param field
     */
    private void doConst(T one, Field field) {
        try {
            LinkConst linkConst = field.getAnnotation(LinkConst.class);
            if (linkConst != null) {
                Class clazz = linkConst.clazz();
                Object object = clazz.newInstance();
                Method getValueMethod = clazz.getDeclaredMethod("getValue", Integer.class);
                if (getValueMethod == null) {
                    throw new RuntimeException("没有获取到getValue方法");
                }
                getValueMethod.setAccessible(true);
                Field declaredField = one.getClass().getDeclaredField(linkConst.fieldName());
                declaredField.setAccessible(true);
                Object getValue = getValueMethod.invoke(object, Integer.valueOf(declaredField.get(one).toString()));
                if (getValue == null) {
                    throw new RuntimeException("常量字典没有这个类");
                }
                ConcurrentHashMap objectObjectHashMap = new ConcurrentHashMap();
                objectObjectHashMap.put("key", Integer.valueOf(declaredField.get(one).toString()));
                objectObjectHashMap.put("value", getValue);
                field.set(one, objectObjectHashMap);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    /**
     * 处理一对多字典情况
     *
     * @param one
     * @param field
     */
    private void doLinkOneToMany(T one, Field field) {
        try {
            LinkMany linkMany = field.getAnnotation(LinkMany.class);
            if (linkMany != null) {
                String linkField = linkMany.linkField();
                Field declaredField = one.getClass().getDeclaredField(linkField);
                declaredField.setAccessible(true);
                Object object = declaredField.get(one);
                Class aClass = linkMany.linkMapper();
                if (object != null) {
                    BaseMapper applicationContextBean = (BaseMapper) this.applicationContext.getBean(aClass);
                    if (applicationContextBean != null) {
                        List<Map> oneToManyResult = applicationContextBean.selectMaps(new QueryWrapper<T>().eq(linkMany.linkPrimaryField(), object));
                        if (oneToManyResult != null && !oneToManyResult.isEmpty()) {
                            field.set(one, oneToManyResult);
                        }
                    } else {
                        return;
                    }
                } else {
                    return;
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            return;
        }

    }

    /**
     * 处理一对一字典情况
     *
     * @param one
     * @param field
     */
    private void doLinkOneToOne(T one, Field field) {
        try {
            LinkOne linkOne = field.getAnnotation(LinkOne.class);
            if (linkOne != null) {
                String linkField = linkOne.linkField();
                Field declaredField = one.getClass().getDeclaredField(linkField);
                declaredField.setAccessible(true);
                Object object = declaredField.get(one);
                Class aClass = linkOne.linkMapper();
                if (object != null) {
                    BaseMapper applicationContextBean = (BaseMapper) this.applicationContext.getBean(aClass);
                    if (applicationContextBean != null) {
                        List<Map> list = applicationContextBean.selectMaps(new QueryWrapper<T>().eq(linkOne.linkPrimaryField(), object));
                        if (list != null && !list.isEmpty()) {
                            Object oneToOneResult = list.get(0);
                            field.set(one, oneToOneResult);
                        }
                    } else {
                        return;
                    }
                } else {
                    return;
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            return;
        }

    }


    public void doManyToMany(T one, Field field) {
        ManyToMany annotation = field.getAnnotation(ManyToMany.class);
        if (annotation == null) {
            return;
        }
        String primaryField = annotation.primaryField();
        try {
            Field declaredField = one.getClass().getDeclaredField(primaryField);
            declaredField.setAccessible(true);
            Object object = declaredField.get(one);
            Class middleMapperClass = annotation.linkMiddleMapper();
            Object midMapper = applicationContext.getBean(middleMapperClass);
            if (midMapper instanceof BaseMapper) {
                BaseMapper<T> middleMapper = (BaseMapper<T>) midMapper;
                String middleField = annotation.middleField();
                List<Object> list = middleMapper.selectObjs(new QueryWrapper<T>().select(middleField).eq(primaryField, object));
                if (list != null && !list.isEmpty()) {
                    Class finalMapperClass = annotation.finalMapper();
                    Object finalMapper = applicationContext.getBean(finalMapperClass);
                    if (finalMapper instanceof BaseMapper) {
                        BaseMapper<T> finallyMapper = (BaseMapper<T>) finalMapper;
                        String finalField = annotation.finalField();
                        List<T> selectList = finallyMapper.selectList(new QueryWrapper<T>().select(finalField).in(finalField, list));
                        field.set(one, selectList);
                    } else {
                        throw new RuntimeException("mapper 转换异常");
                    }

                } else {
                    return;
                }
            } else {
                throw new RuntimeException("mapper 转换异常");
            }

        } catch (Exception e) {
            e.printStackTrace();
        }


    }


    public void doLinkSQL(T one, Object[] params, Field field) throws IllegalAccessException {
        LinkSQL linkSQL = field.getAnnotation(LinkSQL.class);
        if (linkSQL == null) {
            return;
        }
        String value = linkSQL.value();
        if (StringUtils.isBlank(value)) {
            return;
        }
        List<Map<String, Object>> maps = jdbcTemplate.queryForList(value, params);
        if (maps != null && !maps.isEmpty()) {
            field.set(one, maps);
        } else {
            field.set(one, new ArrayList<>());
        }


    }
}
