package edu.travel.web;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.read.listener.ReadListener;
import com.baomidou.mybatisplus.extension.service.IService;
import edu.travel.rpc.RPCBaseResponse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestPart;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.net.URLEncoder;
import java.util.List;

public class BaseController<T> {
    Logger logger = LoggerFactory.getLogger(BaseController.class);
    @Autowired
    private IService<T> service;

    @GetMapping("/download")
    public void download(T clazz, HttpServletResponse response) throws IOException {
        response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
        response.setCharacterEncoding("utf-8");
        String fileName = URLEncoder.encode("导出数据.xlsx", "UTF-8").replaceAll("\\+", "%20");
        response.setHeader("Content-Disposition", "attachment;filename*=utf-8''" + fileName + ".xlsx");
        List<T> list = service.list();
        EasyExcel.write(response.getOutputStream(), clazz.getClass())
                .sheet("数据")
                .doWrite(list);
    }

    @PostMapping("/upload/excel")
    public RPCBaseResponse uploadExcel(@RequestBody T clazz, @RequestPart("file") MultipartFile file) throws IOException {
        EasyExcel.read(file.getInputStream(), clazz.getClass(), new ReadListener<T>() {

                    @Override
                    public void invoke(T t, AnalysisContext analysisContext) {
                        saveTarget(t);
                    }

                    @Override
                    public void doAfterAllAnalysed(AnalysisContext analysisContext) {
                        logger.debug("读取完成");
                    }
                })
                .sheet()
                .doRead();
        return RPCBaseResponse.success();
    }

    @GetMapping("/getById")
    public RPCBaseResponse<T> getId(String id) {
        T byId = service.getById(id);
        return new RPCBaseResponse(HttpStatus.OK.value(), HttpStatus.OK.getReasonPhrase(), byId);
    }

    @PostMapping("/updateById")
    @Transactional
    public RPCBaseResponse<T> updateTargetById(@RequestBody T entity) {
        boolean update = service.updateById(entity);
        if (update) {
            return new RPCBaseResponse<>(200, "success", entity);
        }
        return new RPCBaseResponse(500, "error", entity);
    }

    @PostMapping("/save")
    @Transactional
    public RPCBaseResponse<T> saveTarget(@RequestBody T entity) {
        boolean save = service.save(entity);
        if (save) {
            return new RPCBaseResponse<>(200, "success", entity);
        }
        return new RPCBaseResponse<>(500, "error", entity);
    }

    @PostMapping("/deleteById")
    @Transactional
    public RPCBaseResponse<T> deleteTargetById(@RequestBody List<String> ids) {
        if (ids == null || ids.isEmpty()) {
            return new RPCBaseResponse<>(404, "error,not found delete data", null);
        }
        boolean byIds = service.removeByIds(ids);
        if (byIds) {
            return new RPCBaseResponse<>(200, "success", null);
        }
        return new RPCBaseResponse<>(500, "error, delete data error", null);
    }

    @GetMapping("/list")
    public RPCBaseResponse<List<T>> listAll() {
        List<T> list = service.list();
        if (list.isEmpty()) {
            return new RPCBaseResponse<>(404, "error,not found delete data", null);
        }
        return new RPCBaseResponse<>(200, "success", list);
    }


}
