package edu.travel.remote.upload;

import edu.travel.remote.base.RemoteBaseController;
import edu.travel.remote.upload.dto.EduFileBlobDTO;
import edu.travel.remote.upload.dto.EduFileDTO;
import edu.travel.remote.upload.vo.EduFileVo;
import edu.travel.remote.upload.vo.FileVo;
import edu.travel.rpc.RPCBaseResponse;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.io.Serializable;
import java.util.List;

@FeignClient(name = "upload-dev", path = "/upload")
public interface UploadRemoteController extends RemoteBaseController<EduFileVo, EduFileDTO> {
    @PostMapping("/init")
    public RPCBaseResponse<String> initializeUpload();

    /**
     * 文件是否存在
     *
     * @param md5
     * @return
     */
    @GetMapping("/exits/md5")
    RPCBaseResponse<Serializable> exitsFile(@RequestParam("md5") String md5);

    /**
     * 上传过哪些分片
     *
     * @param md5
     * @return
     */
    @GetMapping("/exits/blob")
    RPCBaseResponse<List<Integer>> exitsBlob(@RequestParam("md5") String md5);

    /**
     * 上传大文件分片
     *
     * @param file
     * @return
     */
    @PostMapping(value = "/uploadBlob", headers = {"content-type=multipart/form-data"})
    RPCBaseResponse uploadBigFile(@RequestBody EduFileBlobDTO dto, @RequestPart("file") MultipartFile file) throws IOException;

    /**
     * 合并大文件
     *
     * @param eduFileDTO
     * @return
     */
    @PostMapping(value = "/mergeFile")
    RPCBaseResponse mergeFile(@RequestBody EduFileDTO eduFileDTO);

    /**
     * 上传小文件
     *
     * @param
     * @return
     */
    @PostMapping(value = "/upload", consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    RPCBaseResponse uploadFile(@RequestParam("eduFileDTO") String dto, @RequestPart("file") MultipartFile file) throws IOException;

    /**
     * 通过id获取文件
     */
    @GetMapping("/getFormId")
    RPCBaseResponse<EduFileVo> getFormId(@RequestParam("id") String id);

    /**
     * 上传小文件
     *
     * @param
     * @return
     */
    @PostMapping(value = "/uploadFilePath", consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    RPCBaseResponse<FileVo> uploadFilePath(@RequestParam("eduFileDTO") String dto, @RequestPart("file") MultipartFile file) throws IOException;


}
