package edu.travel.commodity.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.lang.Snowflake;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import edu.travel.adapter.service.country.CountryAdapter;
import edu.travel.adapter.service.country.CurrencyAdapter;
import edu.travel.adapter.service.order.OrderAdapter;
import edu.travel.adapter.service.order.OrderItemAdapter;
import edu.travel.adapter.service.order.ShopOrderLogAdapter;
import edu.travel.commodity.constant.RedisKey;
import edu.travel.commodity.entity.ShopAddress;
import edu.travel.commodity.entity.ShopProduct;
import edu.travel.commodity.entity.ShopProductSku;
import edu.travel.commodity.entity.ShopSkuSpecValue;
import edu.travel.commodity.enums.OrderStateEnum;
import edu.travel.commodity.mapper.ShopProductSkuMapper;
import edu.travel.commodity.mapper.ShopProductSpecMapper;
import edu.travel.commodity.service.ShopAddressService;
import edu.travel.commodity.service.ShopProductService;
import edu.travel.commodity.service.ShopProductSkuService;
import edu.travel.commodity.service.ShopSkuSpecValueService;
import edu.travel.commodity.utils.TokenData;
import edu.travel.exception.BaseException;
import edu.travel.remote.dto.*;
import edu.travel.remote.vo.*;
import edu.travel.rpc.RPCBaseResponse;
import edu.travel.vo.BaseCountryServeVo;
import edu.travel.vo.ShopCurrencyVo;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;


@Service
public class ShopProductSkuServiceImpl extends ServiceImpl<ShopProductSkuMapper, ShopProductSku> implements ShopProductSkuService {


    @Autowired
    private ShopProductSpecMapper shopProductSpecMapper;
    @Autowired
    private ShopProductService shopProductService;
    @Autowired
    private RedissonClient redissonClient;
    @Autowired
    private ShopProductSkuMapper shopProductSkuMapper;
    @Autowired
    private ShopAddressService shopAddressService;
    @Autowired
    private CountryAdapter countryAdapter;
    @Autowired
    private CurrencyAdapter currencyAdapter;
    @Autowired
    private OrderAdapter orderAdapter;
    @Autowired
    private OrderItemAdapter orderItemAdapter;
    @Autowired
    private ShopOrderLogAdapter shopOrderLogAdapter;
    @Autowired
    private ShopProductSkuService shopProductSkuService;
    @Autowired
    private ShopSkuSpecValueService shopSkuSpecValueService;

    /**
     * 查询SKU下的规格和对应的规格值 规格 排序  规格值排序
     * sku -> 规格 ->规格值
     *
     * @param skuId
     * @return {@link Map }<{@link String },{@link List }<{@link ShopSkuSpecValueVo }>>
     */
    @Override
    public Map<String, List<ShopSkuSpecValueVo>> getSkuValuesById(Set<String> skuId) {
        HttpServletRequest request = ((ServletRequestAttributes) (RequestContextHolder.currentRequestAttributes())).getRequest();
        String project = request.getHeader("project");
        List<ShopSkuSpecValueVo> list = shopProductSpecMapper.getSkuValue(skuId, project);
        return list.stream().collect(Collectors.groupingBy(ShopSkuSpecValueVo::getSkuId));
    }

    @Override
//    @GlobalTransactional
    public void shopProductSku(AddShopOrderDto params) {
        String userId = TokenData.getUserId();
        List<AddShopProductOrderDto> products = params.getProducts();
        //商品ID
        Set<String> product = products.stream().map(AddShopProductOrderDto::getProductId).collect(Collectors.toSet());
        List<ShopProduct> list1 = shopProductService.lambdaQuery().in(ShopProduct::getId, product).eq(ShopProduct::getStatus, 0).list();
        if (list1.size() != product.size()) {
            throw new BaseException("商品不存在");
        }
        HashMap<String, ShopProduct> productMap = new HashMap<>();
        for (ShopProduct shopProduct : list1) {
            productMap.put(shopProduct.getId().toString(), shopProduct);
        }
        List<ShopProductSku> list = shopProductSkuMapper.getByProductSkus(products);
        if (list.size() < products.size()) {
            throw new BaseException("商品规格已下架");
        }
        ShopAddress address = shopAddressService.getById(params.getAddressId());
        if (ObjectUtil.isEmpty(address)) {
            throw new BaseException("地址不存在");
        }
        RPCBaseResponse<ShopCurrencyVo> formId1 = currencyAdapter.getFormId(params.getCurrencyId());
        ShopCurrencyVo currency = formId1.getData();
        RPCBaseResponse<BaseCountryServeVo> formId =
                countryAdapter.getFormId(params.getCountry());
        BaseCountryServeVo country = formId.getData();
        //product+skuId 升序
        List<String> collect = list.stream().map(item -> item.getProductId() + ":" + item.getSkuId()).sorted().collect(Collectors.toList());
        RLock[] locks = new RLock[products.size()];
        for (int i = 0; i < locks.length; i++) {
            String s = collect.get(i);
            locks[i] = redissonClient.getFairLock(RedisKey.PRODUCT_ORDER + s);
        }
        RLock multiLock = redissonClient.getMultiLock(locks);

        try {
            if (multiLock.tryLock(20, TimeUnit.SECONDS)) {
                List<ShopProductSku> ls = shopProductSkuMapper.getByProductSkus(products);
                HashMap<String, ShopProductSku> map = new HashMap<>();
                for (ShopProductSku l : ls) {
                    map.put(l.getProductId().toString() + l.getSkuId(), l);
                }
                BigDecimal all = new BigDecimal(0L);

                ArrayList<AddOrderItemDto> orderItems = new ArrayList<>();
                Snowflake snowflake = IdUtil.createSnowflake(1, 1);
                long orderNum = snowflake.nextId();
                for (AddShopProductOrderDto dto : products) {
                    if (dto.getQuantity() <= 0) {
                        throw new BaseException("购买数量异常");
                    }
                    ShopProduct shopProduct = productMap.get(dto.getProductId());
                    ShopProductSku shopProductSku = map.get(dto.getProductId() + dto.getSkuId());
                    if (ObjectUtil.isEmpty(shopProductSku) || ObjectUtil.isEmpty(shopProduct)) {
                        throw new BaseException("商品" + shopProduct.getProductName() + "已下架");
                    }
                    //冻结量
                    Integer freeze = shopProductSku.getFreeze();
                    //库存
                    Integer inventory = shopProductSku.getInventory();
                    //可用数量
                    int i = inventory - freeze;

                    if (i < dto.getQuantity()) {
                        throw new BaseException(shopProduct.getProductName() + "库存不足");
                    }
                    //修改数量
                    shopProductSku.setFreeze(shopProductSku.getFreeze() + dto.getQuantity());
                    //商品价格
                    BigDecimal multiply = shopProductSku.getPrice().multiply(new BigDecimal(dto.getQuantity()));
                    //总价
                    all = all.add(multiply);
                    //快照信息
                    Map<String, ShopSnapshotVo> snapshot = this.getSnapshot(products);

                    AddOrderItemDto addOrderItemDto = new AddOrderItemDto();
                    addOrderItemDto.setProductId(shopProduct.getId().toString());
                    addOrderItemDto.setSkuId(shopProductSku.getSkuId().toString());
                    addOrderItemDto.setPrice(shopProductSku.getPrice());
                    addOrderItemDto.setQuantity(dto.getQuantity());
                    addOrderItemDto.setOrderId(orderNum + "");

                    ShopSnapshotVo shopSnapshotVo = snapshot.get(shopProductSku.getSkuId().toString());
                    shopSnapshotVo.setProductName(shopProduct.getProductName());
                    shopSnapshotVo.setDescription(shopProduct.getDescription());
                    shopSnapshotVo.setImageUrl(shopProductSku.getImageUrl());
                    addOrderItemDto.setSnapshot(JSONUtil.toJsonStr(shopSnapshotVo));

                    orderItems.add(addOrderItemDto);
                }
                updateBatchById(ls);

                AddOrderDto order = new AddOrderDto();
                order.setCountryId(params.getCountry());
                order.setTotalAmount(all);
                order.setStatus(OrderStateEnum.WAITING_FOR_PAYMENT.getState());
                order.setAddressId(params.getAddressId());
                order.setAddress(address.getDetailedAddress());
                order.setGlobalCouponId(params.getGlobalCouponId());
                order.setServiceChargeValue(country.getServiceChargeValue());
                order.setOrderNumber(orderNum + "");
                order.setCurrency(currency.getCurrencySymbol());
                order.setCurrencyId(currency.getId().toString());
                order.setOrderRemark(params.getRemark());
                order.setUserId(userId);

                orderAdapter.addShopOrder(order);
                orderItemAdapter.addShopOrderItem(orderItems);
                AddOrderLogDto addOrderLogDto = new AddOrderLogDto();
                addOrderLogDto.setOrderId(orderNum + "");
                addOrderLogDto.setDescription("下单成功");
                addOrderLogDto.setStatus(OrderStateEnum.WAITING_FOR_PAYMENT.getState());
                shopOrderLogAdapter.addOrderLog(addOrderLogDto);
            }
        } catch (InterruptedException e) {
            e.printStackTrace();
            throw new BaseException(e);
        } finally {
            multiLock.unlock();//解锁
        }
    }

    @Override
    @Transactional
    public boolean insertProductSku(List<InsertProductSpecDto> specs, List<InsertProductSkuDto> skus, ShopProduct product) {
        ArrayList<ShopProductSku> shopProductSkus = new ArrayList<>();
        ArrayList<ShopSkuSpecValue> shopSkuSpecValues = new ArrayList<>();
        if (ObjectUtil.isEmpty(skus)) {
            ShopProductSku bean = new ShopProductSku();
            bean.setProductId(product.getId());
            Snowflake snowflake = IdUtil.createSnowflake(1, 1);
            long sku = snowflake.nextId();
            bean.setSkuId(sku);
            bean.setPrice(product.getDefaultPrice());
            bean.setImageUrl(product.getMainImageUrl());
            shopProductSkus.add(bean);
            ShopSkuSpecValue skuValue = new ShopSkuSpecValue();
            skuValue.setSkuId(sku);
            skuValue.setSpecValueId(-1L);
            skuValue.setSpecId(-1L);
            shopSkuSpecValues.add(skuValue);
        } else {
            for (InsertProductSkuDto insertProductSkuDto : skus) {
                ShopProductSku bean = BeanUtil.toBean(insertProductSkuDto, ShopProductSku.class);
                bean.setProductId(product.getId());
                bean.setImageUrl(JSONUtil.toJsonStr(insertProductSkuDto.getImageUrl()));
                shopProductSkus.add(bean);
                for (SkuValueDto skuValues : insertProductSkuDto.getSkuValues()) {
                    ShopSkuSpecValue skuValue = new ShopSkuSpecValue();
                    skuValue.setSkuId(bean.getSkuId());
                    List<InsertProductSpecDto> collect = specs.stream().filter(spec -> spec.getSpecName().equals(skuValues.getSpecName())).collect(Collectors.toList());
                    if (ObjectUtil.isEmpty(collect)) {
                        throw new BaseException("规格不存在");
                    }
                    InsertProductSpecDto insertProductSpecDto = collect.get(0);
                    skuValue.setSpecId(Long.valueOf(insertProductSpecDto.getSpecId()));
                    List<SpecValueDto> collect1 = insertProductSpecDto.getSpecValues().stream()
                            .filter(specValue -> specValue.getSpecValueName().equals(skuValues.getSpecValueName())).collect(Collectors.toList());
                    skuValue.setSpecValueId(Long.valueOf(collect1.get(0).getSpecValueId()));

                    shopSkuSpecValues.add(skuValue);
                }
            }
        }
        shopSkuSpecValueService.saveBatch(shopSkuSpecValues);
        return saveBatch(shopProductSkus);
    }

    @Override
//    @GlobalTransactional
    public void payProductSku(PayShopOrderDto payShopOrderDto) {
        //查询订单
        RPCBaseResponse<ShowOrderInfoVo> showOrderInfoVoRPCBaseResponse = orderAdapter.orderInfoByOrderNum(payShopOrderDto.getOrderNum());
        if (showOrderInfoVoRPCBaseResponse.getCode() != 200) {
            throw new BaseException(showOrderInfoVoRPCBaseResponse.getMsg());
        }
        ShowOrderInfoVo data = showOrderInfoVoRPCBaseResponse.getData();
        if (!data.getStatus().equals(OrderStateEnum.WAITING_FOR_PAYMENT.getState())) {
            throw new BaseException("订单状态有误");
        }

        List<OrderItemVo> orderItemVoList = data.getOrderItemVoList();
        List<AddShopProductOrderDto> products = new ArrayList<AddShopProductOrderDto>();
        Map<String, OrderItemVo> itemMap = new ConcurrentHashMap<>();
        for (OrderItemVo orderItemVo : orderItemVoList) {
            itemMap.put(orderItemVo.getProductId() + ":" + orderItemVo.getSkuId(), orderItemVo);
            AddShopProductOrderDto addShopProductOrderDto = new AddShopProductOrderDto();
            addShopProductOrderDto.setProductId(orderItemVo.getProductId());
            addShopProductOrderDto.setSkuId(orderItemVo.getSkuId());
            products.add(addShopProductOrderDto);
        }
        List<String> collect = orderItemVoList.stream().map(item -> item.getProductId() + ":" + item.getSkuId()).sorted()
                .collect(Collectors.toList());
        RLock[] locks = new RLock[collect.size()];
        for (int i = 0; i < locks.length; i++) {
            String s = collect.get(i);
            locks[i] = redissonClient.getFairLock(RedisKey.PRODUCT_ORDER + s);
        }
        RLock multiLock = redissonClient.getMultiLock(locks);
        //扣除冻结和库存
        try {
            if (multiLock.tryLock(20, TimeUnit.SECONDS)) {
                List<ShopProductSku> ls = shopProductSkuMapper.getByProductSkus(products);
                for (ShopProductSku shopProductSku : ls) {
                    OrderItemVo orderItemVo = itemMap.get(shopProductSku.getProductId() + ":" + shopProductSku.getSkuId());
                    String quantity = orderItemVo.getQuantity();
                    int i = Integer.parseInt(quantity);
                    //扣除冻结
                    shopProductSku.setFreeze(shopProductSku.getFreeze() - i);
                    //扣除库存
                    shopProductSku.setInventory(shopProductSku.getInventory() - i);
                }
                updateBatchById(ls);
                // 订单状态扭转
                orderAdapter.changeOrderState(payShopOrderDto.getOrderNum(), OrderStateEnum.AWAITING_RECEIPT.getState() + "");
            }
        } catch (InterruptedException e) {
            e.printStackTrace();
            throw new BaseException(e);
        } finally {
            multiLock.unlock();//解锁
        }
    }


    //获取商品快照信息
    public Map<String, ShopSnapshotVo> getSnapshot(List<AddShopProductOrderDto> products) {
        Set<String> collect = products.stream().map(AddShopProductOrderDto::getSkuId).collect(Collectors.toSet());
        Map<String, List<ShopSkuSpecValueVo>> specs = shopProductSkuService.getSkuValuesById(collect);
        HashMap<String, ShopSnapshotVo> map = new HashMap<>();
        specs.forEach((k, v) -> {
            ShopSnapshotVo shopSnapshotVo = new ShopSnapshotVo();
            shopSnapshotVo.setSkuId(k);
            List<SpecValueVo> specValueVos = BeanUtil.copyToList(v, SpecValueVo.class);
            shopSnapshotVo.setSnapshotSpec(specValueVos);
            map.put(k, shopSnapshotVo);
        });
        return map;
    }


}
