package edu.travel.commodity.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import edu.travel.commodity.constant.BaseConstant;
import edu.travel.commodity.entity.ShopProduct;
import edu.travel.commodity.entity.ShopProductSku;
import edu.travel.commodity.entity.ShopShoppingCart;
import edu.travel.commodity.mapper.ShopProductSpecMapper;
import edu.travel.commodity.mapper.ShopShoppingCartMapper;
import edu.travel.commodity.service.ShopProductService;
import edu.travel.commodity.service.ShopProductSkuService;
import edu.travel.commodity.service.ShopShoppingCartService;
import edu.travel.commodity.utils.PageUtil;
import edu.travel.commodity.utils.TokenData;
import edu.travel.exception.BaseException;
import edu.travel.remote.dto.AddShopCartDto;
import edu.travel.remote.dto.BaseDto;
import edu.travel.remote.dto.UpdateShopCarDto;
import edu.travel.remote.vo.ShopCartListVo;
import edu.travel.remote.vo.ShopSkuSpecValueVo;
import edu.travel.remote.vo.SpecValueVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

@Service
public class ShopShoppingCartServiceImpl extends ServiceImpl<ShopShoppingCartMapper, ShopShoppingCart> implements ShopShoppingCartService {
    @Autowired
    private ShopProductSkuService shopProductSkuService;
    @Autowired
    private ShopProductService shopProductService;
    @Autowired
    private ShopProductSpecMapper shopProductSpecMapper;

    @Override
    public Boolean addShopCart(AddShopCartDto params) {
        if (params.getQuantity() <= 0) {
            throw new BaseException("数量有误，请重新选择");
        }
        // 获取用户id
        String userId = TokenData.getUserId();
        ShopProduct byId = shopProductService.getById(params.getProductId());
        LambdaQueryWrapper<ShopProductSku> query = Wrappers.<ShopProductSku>lambdaQuery().eq(ShopProductSku::getProductId, params.getProductId())
                .eq(ShopProductSku::getSkuId, params.getSkuId());
        ShopProductSku one = shopProductSkuService.getOne(query);
        if (byId == null || byId.getStatus().equals(BaseConstant.BASIC_STATUS_YES_NUM) || one == null) {
            throw new BaseException("商品已经下架");
        }
        ShopShoppingCart shopShoppingCart = new ShopShoppingCart();
        shopShoppingCart.setProductSkuId(one.getId());
        shopShoppingCart.setUserId(Long.valueOf(userId));
        shopShoppingCart.setQuantity(params.getQuantity());
        shopShoppingCart.setProductId(one.getProductId());
        return save(shopShoppingCart);
    }

    @Override
    public Page<ShopCartListVo> getShopCartList(BaseDto params) {
        //登录用户
        String userId = TokenData.getUserId();
        LambdaQueryWrapper<ShopShoppingCart> query = Wrappers.<ShopShoppingCart>lambdaQuery()
                .eq(ShopShoppingCart::getUserId, userId).orderByDesc(ShopShoppingCart::getCreateTime);

        Page<ShopShoppingCart> page = this.page(new Page<ShopShoppingCart>(params.getCurrentPage(), params.getPageSize()), query);
        //转换为结果集
        Page<ShopCartListVo> pageEntity = PageUtil.toPageEntity(page, ShopCartListVo.class);
        //购物车数据
        List<ShopCartListVo> records = pageEntity.getRecords();

        if (ObjectUtil.isEmpty(records)) {
            return pageEntity;
        }

        //商品和sku关联表的id
        Set<String> collect1 = records.stream().map(ShopCartListVo::getProductSkuId).collect(Collectors.toSet());

        List<ShopProductSku> shopProductSkus = shopProductSkuService.listByIds(collect1);
        HashMap<String, ShopProductSku> productSkuMap = new HashMap<>();
        for (ShopProductSku shopProduct : shopProductSkus) {
            productSkuMap.put(shopProduct.getId().toString(), shopProduct);
        }
        // 商品ID
        Set<String> collect = shopProductSkus.stream().map(item -> item.getProductId().toString()).collect(Collectors.toSet());
        Set<String> skus = shopProductSkus.stream().map(item -> item.getSkuId().toString()).collect(Collectors.toSet());
        HttpServletRequest request = ((ServletRequestAttributes) (RequestContextHolder.currentRequestAttributes())).getRequest();
        String project = request.getHeader("project");
        List<ShopSkuSpecValueVo> skuValue = shopProductSpecMapper.getSkuValue(skus, project);
        Map<String, List<ShopSkuSpecValueVo>> sku = skuValue.stream().collect(Collectors.groupingBy(ShopSkuSpecValueVo::getSkuId));
        if (ObjectUtil.isEmpty(collect)) {
            return pageEntity;
        }

        List<ShopProduct> shopProducts = shopProductService.listByIds(collect);
        HashMap<String, ShopProduct> map = new HashMap<>();
        for (ShopProduct shopProduct : shopProducts) {
            map.put(shopProduct.getId().toString(), shopProduct);
        }
        //封装结果集
        for (ShopCartListVo record : records) {
            ShopProduct shopProduct = map.get(record.getProductId());
            if (ObjectUtil.isNotEmpty(shopProduct)) {
                record.setProductState(shopProduct.getStatus().toString());
                record.setProductName(shopProduct.getProductName());
                record.setDescription(shopProduct.getDescription());
                ShopProductSku shopProductSku = productSkuMap.get(record.getProductSkuId());
                if (ObjectUtil.isNotEmpty(shopProductSku)) {
                    record.setSkuImage(shopProductSku.getImageUrl());
                    record.setProductSkuId(shopProductSku.getId().toString());
                    record.setPrice(shopProductSku.getPrice());
                    record.setInventory(shopProductSku.getInventory());
                    record.setSkuId(shopProductSku.getSkuId().toString());
                    //0 上架 1 下架
                    record.setState(shopProductSku.getStatus().toString());
                }
                String skuId = record.getSkuId();
                if (ObjectUtil.isEmpty(skuId)) continue;
                List<ShopSkuSpecValueVo> shopSkuSpecValueVos = sku.get(skuId);
                if (!ObjectUtil.isEmpty(shopSkuSpecValueVos)) {
                    List<SpecValueVo> specValueVos = BeanUtil.copyToList(shopSkuSpecValueVos, SpecValueVo.class);
                    record.setSpecValue(specValueVos);
                }
            }
        }
        return pageEntity;
    }

    @Override
    public Boolean updateCart(UpdateShopCarDto params) {
        ShopShoppingCart cart = getById(params.getId());
        if (cart == null) {
            return false;
        }
        if (!ObjectUtil.isEmpty(params.getQuery()) && params.getQuery() > 0) {
            cart.setQuantity(params.getQuery());
        }
        if (!ObjectUtil.isEmpty(params.getSkuId())) {
            LambdaQueryWrapper<ShopProductSku> query = Wrappers.<ShopProductSku>lambdaQuery().eq(ShopProductSku::getProductId, cart.getProductId())
                    .eq(ShopProductSku::getSkuId, params.getSkuId());
            ShopProductSku one = shopProductSkuService.getOne(query);
            cart.setProductSkuId(one.getId());
        }
        return updateById(cart);
    }

    @Override
    public Integer delCart(Set<String> ids) {
        return baseMapper.deleteBatchIds(ids);
    }


}
