package edu.travel.commodity.web;

import edu.travel.commodity.entity.ShopSpecValue;
import edu.travel.commodity.service.ShopSpecValueService;
import edu.travel.remote.commodity.ShopSpecValueRemoteController;
import edu.travel.remote.dto.ShopSpecValueDto;
import edu.travel.remote.vo.ShopSpecValueVo;
import edu.travel.rpc.RPCBaseResponse;
import edu.travel.web.BaseController;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * 规格值表(shop_spec_value)表控制层
 *
 * @author xxxxx
 */
@RestController
@RequestMapping("/shop_spec_value")
public class ShopSpecValueController extends BaseController<ShopSpecValue> implements ShopSpecValueRemoteController {
    /**
     * 服务对象
     */
    @Autowired
    private ShopSpecValueService shopSpecValueService;

    /**
     * 通过id查找规格值
     *
     * @param id
     * @return
     */
    @Override
    @GetMapping("/getFormId")
    public RPCBaseResponse<ShopSpecValueVo> getFormId(@RequestParam("id") String id) {
        RPCBaseResponse<ShopSpecValue> shopSpecValueRPCBaseResponse = super.getId(id);
        RPCBaseResponse<ShopSpecValueVo> shopSpecValueVoRPCBaseResponse = new RPCBaseResponse<>();
        BeanUtils.copyProperties(shopSpecValueRPCBaseResponse, shopSpecValueVoRPCBaseResponse);
        return shopSpecValueVoRPCBaseResponse;
    }

    /**
     * 更新规格值
     * @param entity
     * @return
     */
    @Override
    public RPCBaseResponse<ShopSpecValueVo> updateTargetFormId(ShopSpecValueDto entity) {
        ShopSpecValue shopSpecValue = new ShopSpecValue();
        BeanUtils.copyProperties(entity, shopSpecValue);
        RPCBaseResponse<ShopSpecValue> shopSpecValueRPCBaseResponse = super.updateTargetById(shopSpecValue);
        RPCBaseResponse<ShopSpecValueVo> shopSpecValueVoRPCBaseResponse = new RPCBaseResponse<>();
        BeanUtils.copyProperties(shopSpecValueRPCBaseResponse, shopSpecValueVoRPCBaseResponse);
        return shopSpecValueVoRPCBaseResponse;
    }

    /**
     * 新增规格值
     * @param entity
     * @return
     */
    @Override
    public RPCBaseResponse<ShopSpecValueVo> saveFormTarget(ShopSpecValueDto entity) {
        ShopSpecValue shopSpecValue = new ShopSpecValue();
        BeanUtils.copyProperties(entity, shopSpecValue);
        RPCBaseResponse<ShopSpecValue> shopSpecValueRPCBaseResponse = super.saveTarget(shopSpecValue);
        RPCBaseResponse<ShopSpecValueVo> shopSpecValueVoRPCBaseResponse = new RPCBaseResponse<>();
        BeanUtils.copyProperties(shopSpecValueRPCBaseResponse, shopSpecValueVoRPCBaseResponse);
        return shopSpecValueVoRPCBaseResponse;
    }

    /**
     * 删除规格值
     * @param ids
     * @return
     */
    @Override
    public RPCBaseResponse<ShopSpecValueVo> deleteTargetFormId(List<String> ids) {
        RPCBaseResponse<ShopSpecValue> shopSpecValueRPCBaseResponse = super.deleteTargetById(ids);
        RPCBaseResponse<ShopSpecValueVo> shopSpecValueVoRPCBaseResponse = new RPCBaseResponse<>();
        BeanUtils.copyProperties(shopSpecValueRPCBaseResponse, shopSpecValueVoRPCBaseResponse);
        return shopSpecValueVoRPCBaseResponse;
    }

    /**
     * 获取所有规格值
     * @return
     */
    @Override
    public RPCBaseResponse<List<ShopSpecValueVo>> getAllForm() {
        RPCBaseResponse<List<ShopSpecValue>> shopSpecValueRPCBaseResponse = super.listAll();
        RPCBaseResponse<List<ShopSpecValueVo>> shopSpecValueVoRPCBaseResponse = new RPCBaseResponse<>();
        BeanUtils.copyProperties(shopSpecValueRPCBaseResponse, shopSpecValueVoRPCBaseResponse);
        return shopSpecValueVoRPCBaseResponse;
    }
}
