package edu.travel.order.controller;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.conditions.query.LambdaQueryChainWrapper;
import edu.travel.exception.BaseException;
import edu.travel.order.entity.ShopOrder;
import edu.travel.order.entity.ShopOrderItem;
import edu.travel.order.enums.OrderStateEnum;
import edu.travel.order.service.ShopOrderItemService;
import edu.travel.order.service.ShopOrderService;
import edu.travel.remote.dto.AddOrderItemDto;
import edu.travel.remote.dto.GetOrderItemDto;
import edu.travel.remote.order.ShopOrderItemRemoteController;
import edu.travel.remote.vo.OrderItemVo;
import edu.travel.remote.vo.ShopSnapshotVo;
import edu.travel.rpc.RPCBaseResponse;
import edu.travel.web.BaseController;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

import static edu.travel.rpc.RPCBaseResponse.success;


@RestController
@RequestMapping("/shopOrderItem")
public class ShopOrderItemController extends BaseController<ShopOrderItem> implements ShopOrderItemRemoteController {

    @Autowired
    private ShopOrderItemService shopOrderItemService;
    @Autowired
    private ShopOrderService shopOrderService;

    /**
     * 获取订单商品详情
     *
     * @param params
     * @return {@link RPCBaseResponse }<{@link OrderItemVo }>
     */

    @Override
    @PostMapping("/getOrderItemVo")
    public RPCBaseResponse<OrderItemVo> getOrderItemVo(@RequestBody GetOrderItemDto params) {
        LambdaQueryWrapper<ShopOrder> orderQuery = Wrappers.<ShopOrder>lambdaQuery().eq(ShopOrder::getOrderNumber, params.getOrderId())
                .eq(ShopOrder::getStatus, OrderStateEnum.AWAITING_EVALUATION.getState());
        ShopOrder one = shopOrderService.getOne(orderQuery);
        if (ObjectUtil.isEmpty(one)) {
            throw new BaseException("订单还未完结不能评价");
        }
        LambdaQueryChainWrapper<ShopOrderItem> query = shopOrderItemService.lambdaQuery().eq(ShopOrderItem::getSkuId, params.getSkuId())
                .eq(ShopOrderItem::getOrderId, params.getOrderId())
                .eq(ShopOrderItem::getProductId, params.getProductId());
        ShopOrderItem shopOrderItem = query.one();
        OrderItemVo bean = BeanUtil.toBean(shopOrderItem, OrderItemVo.class);
        if (!ObjectUtil.isEmpty(shopOrderItem) && !ObjectUtil.isEmpty(shopOrderItem.getSnapshot())) {
            ShopSnapshotVo bean1 = JSONUtil.toBean(shopOrderItem.getSnapshot(), ShopSnapshotVo.class);
            bean.setShopSnapshotVo(bean1);
        }
        return success(bean);
    }

    /**
     * 添加订单商品信息
     *
     * @param params
     * @return {@link RPCBaseResponse }<{@link Void }>
     */

    @Override
    @PostMapping("/addShopOrderItem")
    public RPCBaseResponse<Void> addShopOrderItem(@RequestBody List<AddOrderItemDto> params) {
        shopOrderItemService.addShopOrderItem(params);
        return success();
    }


}
