package edu.travel.upload.web;

import cn.hutool.core.bean.BeanUtil;
import com.alibaba.fastjson.JSON;
import com.obs.services.ObsClient;
import com.obs.services.model.CompleteMultipartUploadRequest;
import com.obs.services.model.CompleteMultipartUploadResult;
import com.obs.services.model.HeaderResponse;
import com.obs.services.model.PutObjectResult;
import edu.travel.remote.upload.UploadRemoteController;
import edu.travel.remote.upload.dto.EduFileBlobDTO;
import edu.travel.remote.upload.dto.EduFileDTO;
import edu.travel.remote.upload.vo.EduFileVo;
import edu.travel.remote.upload.vo.FileVo;
import edu.travel.rpc.RPCBaseResponse;
import edu.travel.upload.config.MD5Calculator;
import edu.travel.upload.entity.EduFile;
import edu.travel.upload.entity.EduFileBlob;
import edu.travel.upload.obs.property.ObsProperties;
import edu.travel.upload.service.EduFileBlobService;
import edu.travel.upload.service.EduFileService;
import edu.travel.web.BaseController;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.io.Serializable;
import java.util.List;
import java.util.UUID;

@RestController
@RequestMapping("/upload")
public class UploadController extends BaseController<EduFile> implements UploadRemoteController {
    //    @Autowired
//    private SysProjectService projectService;
    Logger logger = LoggerFactory.getLogger(UploadController.class);
    @Autowired
    private EduFileService eduFileService;
    @Autowired
    private EduFileBlobService eduFileBlobService;
    @Autowired
    private ObsClient obsClient;
    @Autowired
    private ObsProperties obsProperties;

    @Override
    @PostMapping("/init")
    public RPCBaseResponse<String> initializeUpload() {
        String uploadId = UUID.randomUUID().toString() + System.currentTimeMillis();
        return RPCBaseResponse.success(null, uploadId);
    }

    /**
     * 文件是否存在
     *
     * @param md5
     * @return
     */
    @Override
    @GetMapping("/exits/md5")
    @PreAuthorize("permitAll()")
    public RPCBaseResponse<Serializable> exitsFile(@RequestParam("md5") String md5) {
        return eduFileService.exitsFile(md5);
    }

    /**
     * 上传过哪些分片
     *
     * @param md5
     * @return
     */
    @Override
    @GetMapping("/exits/blob")
    @PreAuthorize("permitAll()")
    public RPCBaseResponse<List<Integer>> exitsBlob(@RequestParam("md5") String md5) {
        return eduFileBlobService.exitsBlob(md5);
    }

    /**
     * 上传大文件分片
     *
     * @param file
     * @return
     */
    @Override
    @PostMapping(value = "/uploadBlob", headers = {"content-type=multipart/form-data"})
    @PreAuthorize("permitAll()")
    public RPCBaseResponse uploadBigFile(EduFileBlobDTO eduFileBlob, MultipartFile file) throws IOException {
        byte[] bytes = file.getBytes();
        String calculateMD5 = MD5Calculator.calculateMD5(bytes);
        if (!eduFileBlob.getBlobMd5().equals(calculateMD5)) {
            return RPCBaseResponse.error("分片md5不一致");
        }
        EduFileBlob fileBlob = new EduFileBlob();
        BeanUtils.copyProperties(eduFileBlob, fileBlob);
        boolean exists = obsClient.headBucket(obsProperties.getBucketName());
        if (!exists) {
            // 若不存在，则创建桶
            HeaderResponse response = obsClient.createBucket(obsProperties.getBucketName());
            logger.info("创建桶成功" + response.getRequestId());
        }

        obsClient.putObject(obsProperties.getBucketName(), "/service/chunk/" + eduFileBlob.getUploadId() + "/" + fileBlob.getBlobNum(), file.getInputStream());
        boolean save = eduFileBlobService.save(fileBlob);
        if (save) {
            return RPCBaseResponse.success();
        }
        return RPCBaseResponse.error();

    }

    /**
     * 合并大文件
     *
     * @param eduFileDTO
     * @return
     */
    @Override
    @PostMapping(value = "/mergeFile")
    @PreAuthorize("permitAll()")
    public RPCBaseResponse mergeFile(EduFileDTO eduFileDTO) {
        CompleteMultipartUploadRequest completeMultipartUploadRequest = new CompleteMultipartUploadRequest(obsProperties.getBucketName(),
                "/service/chunk/" + eduFileDTO.getUploadId(), eduFileDTO.getUploadId(), null);
        CompleteMultipartUploadResult completeMultipartUploadResult = obsClient.completeMultipartUpload(completeMultipartUploadRequest);
        String objectUrl = completeMultipartUploadResult.getObjectUrl();
        EduFile eduFile = new EduFile();
        BeanUtils.copyProperties(eduFileDTO, eduFile);
        eduFile.setFilePath(objectUrl);
        boolean save = eduFileService.save(eduFile);
        if (save) {
            return RPCBaseResponse.success(eduFile.getId().toString());
        }
        return RPCBaseResponse.error();
    }

    /**
     * 上传小文件
     *
     * @param
     * @return
     */
    @Override
    @PostMapping(value = "/upload", consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    @PreAuthorize("permitAll()")
    public RPCBaseResponse uploadFile(@RequestParam("eduFileDTO") String dto, @RequestPart("file") MultipartFile file) throws IOException {
        EduFileDTO eduFileDTO = JSON.parseObject(dto, EduFileDTO.class);
        PutObjectResult putObjectResult = obsClient.putObject(obsProperties.getBucketName(), "/service/chunk/" + eduFileDTO.getUploadId() + "/" + file.getOriginalFilename(), file.getInputStream());
        String objectUrl = putObjectResult.getObjectUrl();
        EduFile files = new EduFile();
        BeanUtils.copyProperties(eduFileDTO, files);
        files.setFilePath(objectUrl);

        boolean save = eduFileService.save(files);
        if (save) {
            return RPCBaseResponse.success(files.getId());
        }
        return RPCBaseResponse.error();
    }

    /**
     * 上传小文件
     *
     * @param
     * @return
     */
    @Override
    @PostMapping(value = "/uploadFilePath", consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    @PreAuthorize("permitAll()")
    public RPCBaseResponse<FileVo> uploadFilePath(@RequestParam("eduFileDTO") String dto, @RequestPart("file") MultipartFile file) throws IOException {
        EduFileDTO eduFileDTO = JSON.parseObject(dto, EduFileDTO.class);
        PutObjectResult putObjectResult = obsClient.putObject(obsProperties.getBucketName(), "/service/chunk/" + eduFileDTO.getUploadId() + "/" + file.getOriginalFilename(), file.getInputStream());
        String objectUrl = putObjectResult.getObjectUrl();
        EduFile files = new EduFile();
        BeanUtils.copyProperties(eduFileDTO, files);
        files.setFilePath(objectUrl);

        boolean save = eduFileService.save(files);
        if (save) {
            return RPCBaseResponse.success(BeanUtil.toBean(files, FileVo.class));
        }
        return RPCBaseResponse.error();
    }


    /**
     * 通过id查找上传文件
     *
     * @param id
     * @return
     */
    @Override
    @GetMapping("/getFormId")
    public RPCBaseResponse<EduFileVo> getFormId(@RequestParam("id") String id) {
        RPCBaseResponse<EduFile> eduFileRPCBaseResponse = super.getId(id);
        RPCBaseResponse<EduFileVo> eduFileVoRPCBaseResponse = new RPCBaseResponse<>();
        BeanUtils.copyProperties(eduFileRPCBaseResponse, eduFileVoRPCBaseResponse);
        return eduFileVoRPCBaseResponse;
    }

    /**
     * 更新文件
     *
     * @param entity
     * @return
     */
    @Override
    @GetMapping("/updateTargetFormId")
    public RPCBaseResponse<EduFileVo> updateTargetFormId(@RequestBody EduFileDTO entity) {
        EduFile eduFile = new EduFile();
        BeanUtils.copyProperties(entity, eduFile);
        RPCBaseResponse<EduFile> eduFileRPCBaseResponse = super.updateTargetById(eduFile);
        RPCBaseResponse<EduFileVo> eduFileVoRPCBaseResponse = new RPCBaseResponse<>();
        BeanUtils.copyProperties(eduFileRPCBaseResponse, eduFileVoRPCBaseResponse);
        return eduFileVoRPCBaseResponse;
    }

    /**
     * 新增文件
     *
     * @param entity
     * @return
     */
    @Override
    @GetMapping("/saveFormTarget")
    public RPCBaseResponse<EduFileVo> saveFormTarget(@RequestBody EduFileDTO entity) {
        EduFile eduFile = new EduFile();
        BeanUtils.copyProperties(entity, eduFile);
        RPCBaseResponse<EduFile> eduFileRPCBaseResponse = super.saveTarget(eduFile);
        RPCBaseResponse<EduFileVo> eduFileVoRPCBaseResponse = new RPCBaseResponse<>();
        BeanUtils.copyProperties(eduFileRPCBaseResponse, eduFileVoRPCBaseResponse);
        return eduFileVoRPCBaseResponse;
    }

    /**
     * 删除文件
     *
     * @param ids
     * @return
     */
    @Override
    @GetMapping("/deleteTargetFormId")
    public RPCBaseResponse<EduFileVo> deleteTargetFormId(@RequestBody List<String> ids) {
        RPCBaseResponse<EduFile> eduFileRPCBaseResponse = super.deleteTargetById(ids);
        RPCBaseResponse<EduFileVo> eduFileVoRPCBaseResponse = new RPCBaseResponse<>();
        BeanUtils.copyProperties(eduFileRPCBaseResponse, eduFileVoRPCBaseResponse);
        return eduFileVoRPCBaseResponse;
    }

    /**
     * 获取全部文件
     *
     * @return
     */
    @Override
    public RPCBaseResponse<List<EduFileVo>> getAllForm() {
        RPCBaseResponse<List<EduFile>> eduFileRPCBaseResponse = super.listAll();
        RPCBaseResponse<List<EduFileVo>> eduFileVoRPCBaseResponse = new RPCBaseResponse<>();
        BeanUtils.copyProperties(eduFileRPCBaseResponse, eduFileVoRPCBaseResponse);
        return eduFileVoRPCBaseResponse;
    }

//    /**
//     * 获取开心中文腾讯会议下载链接
//     */
//    @Override
//    public RPCBaseResponse<String> getTencentMeetingDownloadUrl() {
//
//        eduFileService.getOne(new LambdaQueryWrapper<EduFile>().eq(EduFile.getProject(),"开心中文腾讯会议"))
//        return RPCBaseResponse.success(url);
//    }

}
