package edu.travel.warehouse.web;

import com.baomidou.mybatisplus.core.metadata.IPage;
import edu.travel.dto.ShopWarehouseDto;
import edu.travel.dto.ShopWarehouseSaveDto;
import edu.travel.remote.ShopWarehouseRemoteController;
import edu.travel.rpc.RPCBaseResponse;
import edu.travel.vo.ShopWarehouseStaffVo;
import edu.travel.vo.ShopWarehouseVo;
import edu.travel.warehouse.entity.ShopWarehouse;
import edu.travel.warehouse.service.ShopWarehouseService;
import edu.travel.web.BaseController;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.openfeign.SpringQueryMap;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * 库房表（shop_warehouse)
 */

@RestController
@RequestMapping("/shopWarehouse")
public class ShopWarehouseController extends BaseController<ShopWarehouse> implements ShopWarehouseRemoteController {
    /**
     * 库房表控制层
     */
    @Autowired
    private ShopWarehouseService shopWarehouseService;

    /**
     * 树形结构查询（连表）
     */
    @GetMapping("/getWarehouseCountryForm")
    public RPCBaseResponse<List<ShopWarehouseVo>> getWarehouseCountryForm(ShopWarehouseDto dto) {
        return shopWarehouseService.getWarehouseCountryForm(dto);
    }

    /**
     * 分页动态查询(连表)
     */
    @GetMapping("/getWarehouseCountryPage")
    public RPCBaseResponse<IPage<ShopWarehouseVo>> getWarehouseCountryPage(ShopWarehouseDto dto) {
        return shopWarehouseService.getWarehouseCountryPageForm(dto);
    }

    /**
     * 库房关联人员(角色）查询
     */
    @GetMapping("/getWarehouseTenantRole")
    public RPCBaseResponse<List<ShopWarehouseStaffVo>> getWarehouseTenantRole(String id) {
        return shopWarehouseService.getWarehouseTenantRole(id);
    }

    /**
     * 库房连表新增
     */
    @PostMapping("/saveAllForm")
    public RPCBaseResponse<ShopWarehouseVo> saveAllForm(@RequestBody ShopWarehouseSaveDto entity) {
        HttpServletRequest request = ((ServletRequestAttributes) (RequestContextHolder.currentRequestAttributes())).getRequest();

        // 从请求头中获取 project 的值并设置到 DTO 中
        String project = request.getHeader("Project");
        entity.setProject(project); // 设置项目到 DTO 中
        return shopWarehouseService.saveAllForm(entity); // 只传递 DTO
    }



    /**
     * 库房连表修改
     */
    @PostMapping("/updateWarehouse")
    public RPCBaseResponse<ShopWarehouseVo> updateWarehouse(@RequestBody ShopWarehouseSaveDto entity) {
        HttpServletRequest request = ((ServletRequestAttributes) (RequestContextHolder.currentRequestAttributes())).getRequest();
        // 从请求头中获取 project 的值并设置到 DTO 中
        String project = request.getHeader("Project");
        entity.setProject(project); // 将 project 设置到 DTO 中
        return shopWarehouseService.updateWarehouse(entity); // 只传递 DTO
    }
    /**
     * 查询库房详情
     */
    @Override
    @GetMapping("/getFormId")
    public RPCBaseResponse<ShopWarehouseVo> getFormId(String id) {
        RPCBaseResponse<ShopWarehouse> shopWarehouseRPCBaseResponse = super.getId(id);
        RPCBaseResponse<ShopWarehouseVo> shopWarehouseVoRPCBaseResponse = new RPCBaseResponse<>();
        BeanUtils.copyProperties(shopWarehouseRPCBaseResponse, shopWarehouseVoRPCBaseResponse);
        return shopWarehouseVoRPCBaseResponse;
    }

    /**
     * 更新库房信息
     */
    @Override
    @PostMapping("/updateTargetFormId")
    public RPCBaseResponse<ShopWarehouseVo> updateTargetFormId(@RequestBody ShopWarehouseDto entity) {
        ShopWarehouse shopWarehouse = new ShopWarehouse();
        BeanUtils.copyProperties(entity, shopWarehouse);
        RPCBaseResponse<ShopWarehouse> shopWarehouseRPCBaseResponse = super.updateTargetById(shopWarehouse);
        RPCBaseResponse<ShopWarehouseVo> shopWarehouseVoRPCBaseResponse = new RPCBaseResponse<>();
        BeanUtils.copyProperties(shopWarehouseRPCBaseResponse, shopWarehouseVoRPCBaseResponse);
        return shopWarehouseVoRPCBaseResponse;
    }

    /**
     * 新增库房信息
     */
    @Override
    @PostMapping("/saveFormTarget")
    public RPCBaseResponse<ShopWarehouseVo> saveFormTarget(@RequestBody ShopWarehouseDto entity) {
        ShopWarehouse shopWarehouse = new ShopWarehouse();
        BeanUtils.copyProperties(entity, shopWarehouse);
        RPCBaseResponse<ShopWarehouse> shopWarehouseRPCBaseResponse = super.saveTarget(shopWarehouse);
        RPCBaseResponse<ShopWarehouseVo> shopWarehouseVoRPCBaseResponse = new RPCBaseResponse<>();
        BeanUtils.copyProperties(shopWarehouseRPCBaseResponse, shopWarehouseVoRPCBaseResponse);
        return shopWarehouseVoRPCBaseResponse;
    }

    /**
     * 删除库房信息
     */
    @Override
    @PostMapping("/deleteTargetFormId")
    public RPCBaseResponse<ShopWarehouseVo> deleteTargetFormId(@RequestBody List<String> ids) {
        RPCBaseResponse<ShopWarehouse> shopWarehouseRPCBaseResponse = super.deleteTargetById(ids);
        RPCBaseResponse<ShopWarehouseVo> shopWarehouseVoRPCBaseResponse = new RPCBaseResponse<>();
        BeanUtils.copyProperties(shopWarehouseRPCBaseResponse, shopWarehouseVoRPCBaseResponse);
        return shopWarehouseVoRPCBaseResponse;
    }

    /**
     * 查询所有库房信息
     */
    @Override
    @GetMapping("/getAllForm")
    public RPCBaseResponse<List<ShopWarehouseVo>> getAllForm() {
        RPCBaseResponse<List<ShopWarehouse>> shopWarehouseRPCBaseResponse = super.listAll();
        RPCBaseResponse<List<ShopWarehouseVo>> shopWarehouseVoRPCBaseResponse = new RPCBaseResponse<>();
        BeanUtils.copyProperties(shopWarehouseRPCBaseResponse, shopWarehouseVoRPCBaseResponse);
        return shopWarehouseVoRPCBaseResponse;
    }
}
